<?php
defined('BASEPATH') or exit('No direct script access allowed');

/* ============================
     Definição da Classe Pagou_pix_gateway
   ============================ */
class Pagou_pix_gateway extends App_gateway
{
    protected $ci;

    /* ============================
         Método Construtor
       ============================ */
    public function __construct()
    {
        parent::__construct();

        // Obtém a instância do CodeIgniter
        $this->ci = &get_instance();
        $this->ci->load->database();

        // Configurações do Gateway
        $this->setId('pagou_pix');
        $this->setName('Pagou no PIX');

        // Define as configurações do gateway
        $this->setSettings([
            [
                'name' => 'api_token',
                'encrypted' => true,
                'label' => 'API Token Pagou.com.br',
                'type' => 'input'
            ],
            [
                'name' => 'environment',
                'label' => 'Ambiente',
                'type' => 'select',
                'options' => [
                    'sandbox' => 'Sandbox',
                    'production' => 'Produção'
                ],
                'default_value' => 'production'
            ],
            [
                'name' => 'discount_value',
                'label' => 'Valor do Desconto',
                'type' => 'input'
            ],
            [
                'name' => 'discount_type',
                'label' => 'Tipo de desconto',
                'type' => 'select',
                'options' => [
                    'fixed' => 'Fixo',
                    'percentage' => 'Porcentagem'
                ]
            ],
            [
                'name' => 'discount_days',
                'label' => 'Desconto aplicado para pagamentos realizados com quantos dias de antecedência?',
                'type' => 'input'
            ],
            [
                'name' => 'fine_percentage',
                'label' => 'Multa por atraso %',
                'type' => 'input'
            ],
            [
                'name' => 'interest_percentage',
                'label' => 'Juros por Atraso % ao dia',
                'type' => 'input'
            ]
        ]);

        // Adiciona filtro para gateways de pagamento
        hooks()->add_filter('app_payment_gateways', [$this, 'initMode']);
    }

    /* ============================
         Método process_payment
       ============================ */
    public function process_payment($data)
    {
        // Verifica se a fatura foi passada corretamente
        if (empty($data['invoice'])) {
            return;
        }

        $invoice = $data['invoice'];
        $today = new DateTime();
        $due_date = new DateTime($invoice->duedate);

        // Calcula o valor inicial
        $amount = $invoice->total;

        // Aplica desconto se aplicável
        if ($this->getSetting('discount_value') && $this->getSetting('discount_type')) {
            $discount_days = (int) $this->getSetting('discount_days');
            $days_to_due = $due_date->diff($today)->days;
            if ($today <= $due_date && $days_to_due <= $discount_days) {
                if ($this->getSetting('discount_type') === 'percentage') {
                    $amount -= $amount * ((float)$this->getSetting('discount_value') / 100);
                } else {
                    $amount -= (float)$this->getSetting('discount_value');
                }
            }
        }

        // Aplica juros e multa se aplicável
        if ($today > $due_date) {
            $days_late = $today->diff($due_date)->days;
            if ($this->getSetting('fine_percentage')) {
                $amount += $invoice->total * ((float)$this->getSetting('fine_percentage') / 100);
            }
            if ($this->getSetting('interest_percentage')) {
                $amount += $invoice->total * ((float)$this->getSetting('interest_percentage') / 100) * $days_late;
            }
        }

        // Garante que o valor esteja no formato correto
        $formatted_amount = round($amount, 2);

        // Verifica se existe QR Code gerado não vencido
        $existing_transaction = $this->ci->db->get_where(db_prefix() . 'pagou_transactions_pix', [
            'invoice_id' => $invoice->id,
            'status !=' => 'expired'
        ])->row();

        if ($existing_transaction) {
            $data = [
                'invoice' => $invoice,
                'qr_code' => $existing_transaction->qr_code,
                'qr_code_text' => $existing_transaction->qr_code_text,
                'date_created' => $existing_transaction->date_created,
                'expiration' => '24 horas'
            ];
            $this->ci->load->view('pagou/payment_pix', $data);
            return;
        }

        // Prepara os dados para enviar
        $payment_data = [
            'amount' => $formatted_amount,
            'description' => 'Fatura #' . format_invoice_number($invoice->id),
            'expiration' => 86400, // 24 horas
            'notification_url' => site_url('pagou/notification'),
            'payer' => [
                'name' => $invoice->client->company,
                'document' => preg_replace('/[^0-9]/', '', $invoice->client->vat)
            ]
        ];

        // Requisição para API
        $api_endpoint = $this->getSetting('environment') == 'sandbox'
            ? 'https://api.pagou.com.br/v1/pix'
            : 'https://api.pagou.com.br/v1/pix';

        $ch = curl_init($api_endpoint);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($payment_data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'X-API-KEY: ' . $this->decryptSetting('api_token')
        ]);

        $response = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($http_code === 201) {
            $result = json_decode($response);
            $this->ci->db->insert(db_prefix() . 'pagou_transactions_pix', [
                'invoice_id' => $invoice->id,
                'cliente' => $invoice->client->company,
                'doc' => preg_replace('/[^0-9]/', '', $invoice->client->vat),
                'transaction_id' => $result->id ?? '',
                'payment_method' => 'pix',
                'qr_code' => $result->payload->image,
                'qr_code_text' => $result->payload->data,
                'amount' => $invoice->total,
                'amount_final' => $formatted_amount,
                'status' => $result->status ?? '',
                'date_created' => date('Y-m-d H:i:s')
            ]);

            $data = [
                'invoice' => $invoice,
                'qr_code' => $result->payload->image,
                'qr_code_text' => $result->payload->data,
                'date_created' => date('Y-m-d H:i:s'),
                'expiration' => '24 horas'
            ];
            $this->ci->load->view('pagou/payment_pix', $data);
        } else {
            set_alert('danger', 'Erro ao gerar PIX. Por favor, tente novamente.');
            redirect(site_url('invoice/' . $invoice->id . '/' . $invoice->hash));
        }
    }

    /* ============================
         Método webhook
       ============================ */
    public function webhook()
    {
        // Processa notificações da API
        $data = json_decode(file_get_contents('php://input'), true);
        if (!empty($data['transaction_id'])) {
            $this->ci->db->where('transaction_id', $data['transaction_id']);
            $this->ci->db->update(db_prefix() . 'pagou_transactions_pix', [
                'status' => $data['status'] ?? '',
                'data_pagamento' => $data['paid_at'] ?? null
            ]);
        }
    }
}
?>
