<?php
defined('BASEPATH') or exit('No direct script access allowed');

/* ============================
     Definição da Classe Api_client
   ============================ */
class Api_client
{
    private $api_token;
    private $api_endpoint = 'https://api.pagou.com.br/v1/charges';

    /**
     * Método Construtor
     *
     * Inicializa o cliente da API com o token fornecido.
     *
     * @param string $api_token Token da API Pagou.com.br
     */
    public function __construct($api_token)
    {
        $this->api_token = $api_token;
        // log_message('debug', 'Api_client inicializado com token: ' . substr($api_token, 0, 8) . '...');
    }

    /**
     * Consulta o status de uma transação específica.
     *
     * @param string $transaction_id ID da transação a ser consultada.
     * @return object|null Retorna o objeto decodificado da resposta ou null em caso de erro.
     */
    public function consultTransaction($transaction_id)
    {
        // log_message('info', 'Iniciando consulta de transação na API Pagou - Transaction ID: ' . $transaction_id);

        if (empty($transaction_id)) {
            // log_message('error', 'Transaction ID vazio ou inválido');
            return null;
        }

        $url = rtrim($this->api_endpoint, '/') . '/' . $transaction_id;
        // log_message('debug', 'URL da consulta: ' . $url);

        /* ============================
             Inicialização do CURL
           ============================ */
        $ch = curl_init();

        /* ============================
             Definição das Opções do CURL
           ============================ */
        $headers = [
            'Accept: application/json',
            'Content-Type: application/json',
            'X-API-KEY: ' . trim($this->api_token)
        ];

        $curl_options = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_SSL_VERIFYHOST => 2,
            CURLOPT_VERBOSE => true,
            CURLOPT_HEADER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_USERAGENT => 'Perfex CRM/Pagou Gateway',
            CURLOPT_CONNECTTIMEOUT => 10,
            CURLOPT_DNS_CACHE_TIMEOUT => 120,
            CURLOPT_IPRESOLVE => CURL_IPRESOLVE_V4
        ];

        curl_setopt_array($ch, $curl_options);

        // log_message('debug', 'Headers da requisição: ' . json_encode($headers));

        /* ============================
             Captura do Output Verbose
           ============================ */
        $verbose = fopen('php://temp', 'w+');
        curl_setopt($ch, CURLOPT_STDERR, $verbose);

        /* ============================
             Execução da Requisição CURL
           ============================ */
        $start_time = microtime(true);
        $response = curl_exec($ch);
        $end_time = microtime(true);

        /* ============================
             Obtenção das Informações da Requisição
           ============================ */
        $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $header = substr($response, 0, $header_size);
        $body = substr($response, $header_size);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curl_error = curl_error($ch);
        $curl_errno = curl_errno($ch);
        $total_time = curl_getinfo($ch, CURLINFO_TOTAL_TIME);
        $connect_time = curl_getinfo($ch, CURLINFO_CONNECT_TIME);
        $name_lookup_time = curl_getinfo($ch, CURLINFO_NAMELOOKUP_TIME);

        /* ============================
             Log do Output Verbose
           ============================ */
        rewind($verbose);
        $verbose_log = stream_get_contents($verbose);
        fclose($verbose);

        /* ============================
             Log Detalhado da Requisição
           ============================ */
        /*
        log_message('debug', sprintf(
            'Detalhes da requisição CURL: 
            - URL: %s
            - HTTP Code: %d
            - Tempo total: %.2fs
            - Tempo de conexão: %.2fs
            - Tempo de DNS lookup: %.2fs
            - Erro: %s
            - Errno: %d
            - Header size: %d bytes
            - Response size: %d bytes
            - Headers recebidos: %s
            - Body recebido: %s
            - Verbose log: %s',
            $url,
            $http_code,
            $total_time,
            $connect_time,
            $name_lookup_time,
            $curl_error ?: 'Nenhum',
            $curl_errno,
            $header_size,
            strlen($body),
            $header,
            $body,
            $verbose_log
        ));
        */

        curl_close($ch);

        /* ============================
             Verificação de Erros no CURL
           ============================ */
        if ($curl_errno) {
            /*
            log_message('error', sprintf(
                'Erro CURL na consulta - Transaction ID: %s, Errno: %d, Error: %s, Verbose: %s',
                $transaction_id,
                $curl_errno,
                $curl_error,
                $verbose_log
            ));
            */
            return null;
        }

        /* ============================
             Verificação do Código HTTP da Resposta
           ============================ */
        if ($http_code === 200) {
            // log_message('info', 'Consulta realizada com sucesso - Transaction ID: ' . $transaction_id);

            /* ============================
                 Decodificação da Resposta JSON
               ============================ */
            $decoded = json_decode($body);
            if (json_last_error() === JSON_ERROR_NONE) {
                // log_message('debug', 'Resposta decodificada com sucesso: ' . json_encode($decoded));
                return $decoded;
            }

            /*
            log_message('error', sprintf(
                'Erro ao decodificar JSON - Error: %s, Response: %s',
                json_last_error_msg(),
                $body
            ));
            */
            return null;
        }

        /* ============================
             Tratamento de Erros na Consulta
           ============================ */
        /*
        log_message('error', sprintf(
            'Erro na consulta - Transaction ID: %s, HTTP Code: %d, Response: %s, Headers: %s, Verbose: %s',
            $transaction_id,
            $http_code,
            $body,
            $header,
            $verbose_log
        ));
        */

        return null;
    }

    /**
     * Cria uma nova cobrança na API Pagou.
     *
     * @param array $payment_data Dados do pagamento a serem enviados para a API.
     * @return array Resultado da operação com 'success' e 'message' ou 'data'.
     */
    public function createCharge($payment_data)
    {
        // log_message('info', 'Iniciando criação de cobrança na API Pagou');
        // log_message('debug', 'Dados do pagamento: ' . json_encode($payment_data));

        /* ============================
             Inicialização do CURL para Criação de Cobrança
           ============================ */
        $ch = curl_init();

        $json_data = json_encode($payment_data);
        // log_message('debug', 'Payload JSON: ' . $json_data);

        $headers = [
            'Accept: application/json',
            'Content-Type: application/json',
            'X-API-KEY: ' . trim($this->api_token)
        ];

        $curl_options = [
            CURLOPT_URL => $this->api_endpoint,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_POST => 1,
            CURLOPT_POSTFIELDS => $json_data,
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_SSL_VERIFYHOST => 2,
            CURLOPT_VERBOSE => true,
            CURLOPT_HEADER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_USERAGENT => 'Perfex CRM/Pagou Gateway',
            CURLOPT_CONNECTTIMEOUT => 10,
            CURLOPT_DNS_CACHE_TIMEOUT => 120,
            CURLOPT_IPRESOLVE => CURL_IPRESOLVE_V4
        ];

        curl_setopt_array($ch, $curl_options);

        // log_message('debug', 'Headers da requisição: ' . json_encode($headers));

        /* ============================
             Captura do Output Verbose
           ============================ */
        $verbose = fopen('php://temp', 'w+');
        curl_setopt($ch, CURLOPT_STDERR, $verbose);

        /* ============================
             Execução da Requisição CURL
           ============================ */
        $start_time = microtime(true);
        $response = curl_exec($ch);
        $end_time = microtime(true);

        /* ============================
             Obtenção das Informações da Requisição
           ============================ */
        $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $header = substr($response, 0, $header_size);
        $body = substr($response, $header_size);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curl_error = curl_error($ch);
        $curl_errno = curl_errno($ch);
        $connect_time = curl_getinfo($ch, CURLINFO_CONNECT_TIME);
        $name_lookup_time = curl_getinfo($ch, CURLINFO_NAMELOOKUP_TIME);

        /* ============================
             Log do Output Verbose
           ============================ */
        rewind($verbose);
        $verbose_log = stream_get_contents($verbose);
        fclose($verbose);

        /* ============================
             Log Detalhado da Requisição
           ============================ */
        /*
        log_message('debug', sprintf(
            'Detalhes da requisição CURL: 
            - URL: %s
            - HTTP Code: %d
            - Tempo total: %.2fs
            - Tempo de conexão: %.2fs
            - Tempo de DNS lookup: %.2fs
            - Erro: %s
            - Errno: %d
            - Header size: %d bytes
            - Response size: %d bytes
            - Headers recebidos: %s
            - Body recebido: %s
            - Verbose log: %s',
            $this->api_endpoint,
            $http_code,
            $end_time - $start_time,
            $connect_time,
            $name_lookup_time,
            $curl_error ?: 'Nenhum',
            $curl_errno,
            $header_size,
            strlen($body),
            $header,
            $body,
            $verbose_log
        ));
        */

        curl_close($ch);

        /* ============================
             Verificação de Erros no CURL
           ============================ */
        if ($curl_errno) {
            /*
            log_message('error', sprintf(
                'Erro CURL na criação - Errno: %d, Error: %s, Verbose: %s',
                $curl_errno,
                $curl_error,
                $verbose_log
            ));
            */
            return [
                'success' => false,
                'message' => 'Erro de conexão: ' . $curl_error
            ];
        }

        /* ============================
             Decodificação da Resposta JSON
           ============================ */
        $response_data = json_decode($body);
        $json_error = json_last_error();

        /* ============================
             Verificação do Código HTTP e Decodificação
           ============================ */
        if ($http_code === 201 || $http_code === 200) {
            if ($json_error === JSON_ERROR_NONE) {
                // log_message('info', 'Cobrança criada com sucesso - ID: ' . ($response_data->id ?? 'N/A'));
                return [
                    'success' => true,
                    'data' => $response_data
                ];
            }

            /*
            log_message('error', sprintf(
                'Erro ao decodificar JSON - Error: %s, Response: %s',
                json_last_error_msg(),
                $body
            ));
            */
        }

        /* ============================
             Tratamento de Erros na Criação da Cobrança
           ============================ */
        $error_message = isset($response_data->error) ? $response_data->error : 'Erro desconhecido';

        /*
        log_message('error', sprintf(
            'Erro na criação da cobrança - HTTP Code: %d, Erro: %s, Response: %s, Headers: %s, Verbose: %s',
            $http_code,
            $error_message,
            $body,
            $header,
            $verbose_log
        ));
        */

        return [
            'success' => false,
            'message' => $error_message
        ];
    }
}
?>
