<?php
defined('BASEPATH') or exit('No direct script access allowed');

/* ============================
     Definição da Classe Payment_processor
   ============================ */
class Payment_processor
{
    private $ci;
    private $transaction_handler;
    private $api_client;
    private $gateway;

    /**
     * Método Construtor
     *
     * Inicializa o processador de pagamentos com as dependências necessárias.
     *
     * @param object $ci Instância do CodeIgniter.
     * @param object $transaction_handler Manipulador de transações.
     * @param object $api_client Cliente da API Pagou.
     * @param object $gateway Gateway de pagamento.
     */
    public function __construct($ci, $transaction_handler, $api_client, $gateway)
    {
        $this->ci = $ci;
        $this->transaction_handler = $transaction_handler;
        $this->api_client = $api_client;
        $this->gateway = $gateway;
    }

    /**
     * Processa o pagamento para uma fatura específica.
     *
     * @param object $invoice Objeto da fatura a ser processada.
     * @return bool Retorna true se o pagamento for processado com sucesso, ou false em caso de falha.
     */
    public function process($invoice)
    {
        /*
        log_message('info', 'Iniciando processamento de pagamento para invoice ID: ' . $invoice->id);
        */

        /* ============================
             FASE 1: Verificar transação existente e atualizar dados
           ============================ */
        /*
        log_message('debug', 'FASE 1: Verificando transação existente');
        */
        $existing_transaction = $this->transaction_handler->checkExistingTransaction($invoice->id);

        if ($existing_transaction) {
            /*
            log_message('info', sprintf(
                'Transação existente encontrada - ID: %s, Status: %s',
                $existing_transaction->transaction_id ?? 'N/A',
                $existing_transaction->status ?? 'N/A'
            ));
            */

            // Se existe transaction_id, verifica se precisa consultar a API
            if (!empty($existing_transaction->transaction_id)) {
                /*
                log_message('debug', 'Verificando necessidade de consulta à API');
                */

                // Se consulta está vazia, null ou 0, faz a consulta na API
                if (empty($existing_transaction->consulta) ||
                    is_null($existing_transaction->consulta) ||
                    $existing_transaction->consulta === '0') {

                    /*
                    log_message('info', 'Consultando dados atualizados na API');
                    */

                    // Consulta a API e atualiza os dados
                    $response = $this->api_client->consultTransaction($existing_transaction->transaction_id);

                    if ($response) {
                        /*
                        log_message('debug', 'Dados recebidos da API: ' . json_encode($response));
                        */

                        // Atualiza os dados da transação
                        $this->transaction_handler->updateTransactionFromApi(
                            $invoice->id,
                            $existing_transaction->transaction_id
                        );

                        // Recarrega a transação com os dados atualizados
                        $existing_transaction = $this->transaction_handler->checkExistingTransaction($invoice->id);
                    } else {
                        /*
                        log_message('error', 'Falha ao consultar API');
                        */
                    }
                } else {
                    /*
                    log_message('debug', 'Consulta à API não necessária');
                    */
                }
            }

            /*
            log_message('info', 'Carregando visualização do boleto existente');
            */
            return $this->loadBoletoView($invoice, $existing_transaction);
        }

        /* ============================
             FASE 2: Criar nova transação
           ============================ */
        /*
        log_message('debug', 'FASE 2: Criando nova transação');
        */

        $client = $this->getClientData($invoice->clientid);
        if (!$client) {
            /*
            log_message('error', 'Cliente não encontrado - ID: ' . $invoice->clientid);
            */
            set_alert('danger', 'Cliente não encontrado');
            redirect(site_url('invoice/' . $invoice->id . '/' . $invoice->hash));
            return false;
        }

        /*
        log_message('debug', 'Preparando dados para nova transação');
        */
        $payment_data = $this->preparePaymentData($invoice, $client);
        /*
        log_message('debug', 'Dados preparados: ' . json_encode($payment_data));
        */

        $response = $this->api_client->createCharge($payment_data);

        if ($response['success']) {
            /*
            log_message('info', 'Nova transação criada com sucesso');
            */

            // Salva a nova transação
            $this->transaction_handler->saveNewTransaction($invoice->id, $response['data']);

            /*
            log_activity('Boleto gerado com sucesso para a fatura ID: ' . $invoice->id);
            */

            // Novo processo:
            // Apresenta uma notificação "Consultando Informações... Aguarde"
            $this->ci->session->set_flashdata('alert-success', 'Consultando Informações... Aguarde');

            $max_attempts = 6;
            $attempts = 0;

            do {
                // Aguardar 5 segundos antes de consultar novamente
                sleep(5);

                // Recarrega a transação
                $transaction = $this->transaction_handler->checkExistingTransaction($invoice->id);

                // Verifica se os campos necessários estão preenchidos
                if ($this->areRequiredFieldsFilled($transaction)) {
                    // Campos preenchidos, carrega a view
                    return $this->loadBoletoView($invoice, $transaction);
                }

                // Caso contrário, apresenta novamente a notificação e consulta a API
                $this->ci->session->set_flashdata('alert-info', 'Consultando Informações... Aguarde');
                $response = $this->api_client->consultTransaction($transaction->transaction_id);

                if ($response) {
                    $this->transaction_handler->updateTransactionFromApi($invoice->id, $transaction->transaction_id);
                }

                $attempts++;
            } while ($attempts < $max_attempts);

            // Caso não seja possível preencher os campos em 6 tentativas
            set_alert('danger', 'Erro ao consultar os dados da Fatura!');
            redirect(site_url('invoice/' . $invoice->id . '/' . $invoice->hash));
            return false;
        }

        /*
        log_message('error', sprintf(
            'Erro ao criar transação - Invoice ID: %d, Erro: %s',
            $invoice->id,
            $response['message']
        ));
        */

        set_alert('danger', $response['message']);
        /*
        log_activity('Pagou Boleto Error: ' . $response['message']);
        */
        redirect(site_url('invoice/' . $invoice->id . '/' . $invoice->hash));
        return false;
    }

    /**
     * Verifica se os campos obrigatórios (qr_code, qr_code_text, qr_code_img) estão preenchidos.
     *
     * @param object $transaction Objeto da transação.
     * @return bool Retorna true se todos os campos estiverem preenchidos, caso contrário false.
     */
    private function areRequiredFieldsFilled($transaction)
    {
        return !empty($transaction->qr_code) && !empty($transaction->qr_code_text) && !empty($transaction->qr_code_img);
    }

    /**
     * Carrega a visualização do boleto existente.
     *
     * @param object $invoice Objeto da fatura.
     * @param object $transaction Objeto da transação existente.
     * @return bool Retorna true se a view for carregada com sucesso, ou false em caso de falha.
     */
    private function loadBoletoView($invoice, $transaction)
    {
        /*
        log_message('debug', sprintf(
            'Carregando view do boleto - Invoice ID: %d, Transaction ID: %s',
            $invoice->id,
            $transaction->transaction_id ?? 'N/A'
        ));
        */

        if (!$transaction) {
            /*
            log_message('error', 'Transação não encontrada ao carregar view');
            */
            set_alert('danger', 'Transação não encontrada');
            redirect(site_url('invoice/' . $invoice->id . '/' . $invoice->hash));
            return false;
        }

        $currency = get_currency($invoice->currency);
        $view_data = [
            'invoice' => $invoice,
            'transaction' => $transaction,
            'transaction_id' => $transaction->transaction_id ?? '',
            'boleto_url' => !empty($transaction->transaction_id) ?
                'https://fatura.pagou.com.br/boleto/' . $transaction->transaction_id : '',
            'barcode' => $transaction->barcode ?? '',
            'qr_code' => $transaction->qr_code ?? '',
            'qr_code_text' => $transaction->qr_code_text ?? '',
            'qr_code_img' => $transaction->qr_code_img ?? '',
            'due_date' => $invoice->duedate,
            'currency_name' => $currency->name,
            'status' => $transaction->status ?? 'pending'
        ];

        /*
        log_message('debug', 'Dados para view: ' . json_encode($view_data));
        */

        $this->ci->load->view('pagou/payment_boleto', $view_data);
        return true;
    }

    /**
     * Obtém os dados do cliente a partir do ID.
     *
     * @param int $client_id ID do cliente.
     * @return object|null Retorna o objeto do cliente ou null se não encontrado.
     */
    private function getClientData($client_id)
    {
        /*
        log_message('debug', 'Obtendo dados do cliente - ID: ' . $client_id);
        */

        $this->ci->db->select('userid, company, vat, phonenumber, billing_street, billing_city, billing_state, billing_zip');
        $this->ci->db->where('userid', $client_id);
        $client = $this->ci->db->get(db_prefix() . 'clients')->row();

        if ($client) {
            /*
            log_message('debug', 'Dados do cliente encontrados: ' . json_encode($client));
            */
        } else {
            /*
            log_message('error', 'Cliente não encontrado');
            */
        }

        return $client;
    }

    /**
     * Prepara os dados de pagamento para serem enviados à API.
     *
     * @param object $invoice Objeto da fatura.
     * @param object $client Objeto do cliente.
     * @return array Retorna um array com os dados de pagamento preparados.
     */
    private function preparePaymentData($invoice, $client)
    {
        /*
        log_message('debug', 'Preparando dados de pagamento');
        */

        // Calcula o valor total
        $amount = $this->calculateAmount($invoice);

        // Validação e limpeza de CPF/CNPJ
        $document = preg_replace('/[^0-9]/', '', $client->vat ?? '');
        if (empty($document) || (strlen($document) !== 11 && strlen($document) !== 14)) {
            /*
            log_message('error', 'CPF/CNPJ inválido: ' . ($client->vat ?? 'Não informado'));
            */
            $document = '00000000000'; // Valor padrão genérico
        }

        // Validação do estado para sigla (SP, RJ, etc.)
        $state = $this->getStateAbbreviation($client->billing_state ?? '');
        if (empty($state) || strlen($state) !== 2) {
            /*
            log_message('error', 'Estado inválido: ' . ($client->billing_state ?? 'Não informado'));
            */
            $state = 'SP'; // Valor padrão
        }

        // Validação do CEP (somente números e com 8 dígitos)
        $postal_code = preg_replace('/[^0-9]/', '', $client->billing_zip ?? '');
        if (strlen($postal_code) !== 8) {
            /*
            log_message('error', 'CEP inválido: ' . ($client->billing_zip ?? 'Não informado'));
            */
            $postal_code = '00000000'; // Valor padrão
        }

        // Validação da rua (remove números e verifica se está preenchido)
        $street = trim(preg_replace('/\d+/', '', $client->billing_street ?? ''));
        if (empty($street)) {
            /*
            log_message('error', 'Rua inválida: ' . ($client->billing_street ?? 'Não informado'));
            */
            $street = 'Endereço não especificado';
        }

        // Validação da cidade
        $city = trim($client->billing_city ?? '');
        if (empty($city)) {
            /*
            log_message('error', 'Cidade inválida: ' . ($client->billing_city ?? 'Não informado'));
            */
            $city = 'Cidade não especificada';
        }

        // Validação do bairro
        $neighborhood = trim($client->neighborhood ?? 'Centro');
        if (empty($neighborhood)) {
            /*
            log_message('error', 'Bairro inválido: ' . ($client->neighborhood ?? 'Não informado'));
            */
            $neighborhood = 'Centro';
        }

        // Validação do número do endereço
        $number = trim($client->address_number ?? 'S/N');
        if (empty($number)) {
            /*
            log_message('error', 'Número inválido: ' . ($client->address_number ?? 'Não informado'));
            */
            $number = 'S/N';
        }

        // Período de carência
        $grace_period = (int)$this->gateway->getSetting('grace_period');
        $grace_period = $grace_period > 0 ? $grace_period : null;

        /* ============================
             Monta o payload de dados
           ============================ */
        $payment_data = [
            'amount' => $amount,
            'description' => 'Fatura #' . format_invoice_number($invoice->id),
            'due_date' => $invoice->duedate,
            'fine' => (float)$this->gateway->getSetting('fine_percentage'),
            'interest' => (float)$this->gateway->getSetting('interest_percentage'),
            'grace_period' => $grace_period,
            'notification_url' => site_url('pagou/webhook'),
            'payer' => [
                'name' => $client->company ?: 'Cliente',
                'document' => $document,
                'street' => $street,
                'number' => $number,
                'neighborhood' => $neighborhood,
                'city' => $city,
                'state' => $state,
                'zip' => $postal_code
            ],
            'metadata' => [
                [
                    'key' => 'invoice_id',
                    'value' => $invoice->id
                ]
            ]
        ];

        /*
        log_message('debug', 'Dados de pagamento preparados: ' . json_encode($payment_data));
        */
        return $payment_data;
    }

    /**
     * Calcula o valor final do pagamento, aplicando descontos se necessário.
     *
     * @param object $invoice Objeto da fatura.
     * @return float Retorna o valor final calculado.
     */
    private function calculateAmount($invoice)
    {
        /*
        log_message('debug', 'Calculando valor final');
        */

        $amount = $invoice->total;
        $discount_value = $this->gateway->getSetting('discount_value');
        $discount_type = $this->gateway->getSetting('discount_type');

        if ($discount_value && $discount_type) {
            /*
            log_message('debug', sprintf(
                'Aplicando desconto - Tipo: %s, Valor: %s',
                $discount_type,
                $discount_value
            ));
            */

            if ($discount_type === 'percentage') {
                $amount = $amount * (1 - ((float)$discount_value / 100));
            } else {
                $amount = $amount - (float)$discount_value;
            }
        }

        $final_amount = round($amount, 2);
        /*
        log_message('debug', sprintf(
            'Valor calculado - Original: %s, Final: %s',
            $invoice->total,
            $final_amount
        ));
        */

        return $final_amount;
    }

    /**
     * Converte o nome do estado para sua sigla correspondente.
     *
     * @param string $state Nome completo do estado.
     * @return string Retorna a sigla do estado ou 'SP' como padrão.
     */
    private function getStateAbbreviation($state)
    {
        $states = [
            'AC' => 'AC', 'AL' => 'AL', 'AP' => 'AP', 'AM' => 'AM',
            'BA' => 'BA', 'CE' => 'CE', 'DF' => 'DF', 'ES' => 'ES',
            'GO' => 'GO', 'MA' => 'MA', 'MT' => 'MT', 'MS' => 'MS',
            'MG' => 'MG', 'PA' => 'PA', 'PB' => 'PB', 'PR' => 'PR',
            'PE' => 'PE', 'PI' => 'PI', 'RJ' => 'RJ', 'RN' => 'RN',
            'RS' => 'RS', 'RO' => 'RO', 'RR' => 'RR', 'SC' => 'SC',
            'SP' => 'SP', 'SE' => 'SE', 'TO' => 'TO'
        ];

        $state = strtoupper(trim($state));
        $result = $states[$state] ?? 'SP';

        /*
        log_message('debug', sprintf(
            'Convertendo estado - Original: %s, Resultado: %s',
            $state,
            $result
        ));
        */

        return $result;
    }
}
