<?php
defined('BASEPATH') or exit('No direct script access allowed');

/* ============================
     Definição da Classe Transaction_handler
   ============================ */
class Transaction_handler
{
    private $ci;
    private $api_client;

    /**
     * Método Construtor
     *
     * Inicializa o manipulador de transações com as dependências necessárias.
     *
     * @param object $ci Instância do CodeIgniter.
     * @param object $api_client Cliente da API Pagou.
     */
    public function __construct($ci, $api_client)
    {
        $this->ci = $ci;
        $this->api_client = $api_client;
    }

    /**
     * Verifica se já existe uma transação para uma fatura específica.
     *
     * @param int $invoice_id ID da fatura.
     * @return object|null Retorna o objeto da transação ou null se não encontrado.
     */
    public function checkExistingTransaction($invoice_id)
    {
        /*
        log_message('debug', 'Verificando transação existente para invoice_id: ' . $invoice_id);
        */
        
        $transaction = $this->ci->db->where('invoice_id', $invoice_id)
                                  ->get(db_prefix() . 'pagou_transactions')
                                  ->row();
        
        if ($transaction) {
            /*
            log_message('info', sprintf(
                'Transação encontrada - Invoice ID: %d, Transaction ID: %s, Status: %s',
                $invoice_id,
                $transaction->transaction_id ?? 'N/A',
                $transaction->status ?? 'N/A'
            ));
            */
        } else {
            /*
            log_message('info', 'Nenhuma transação encontrada para invoice_id: ' . $invoice_id);
            */
        }
        
        return $transaction;
    }

    /**
     * Atualiza uma transação existente com dados obtidos da API.
     *
     * @param int $invoice_id ID da fatura.
     * @param string $transaction_id ID da transação.
     * @return object|null Retorna a transação atualizada ou null em caso de falha.
     */
    public function updateTransactionFromApi($invoice_id, $transaction_id)
    {
        /*
        log_message('info', sprintf(
            'Atualizando transação com dados da API - Invoice ID: %d, Transaction ID: %s',
            $invoice_id,
            $transaction_id
        ));
        */
        
        $response = $this->api_client->consultTransaction($transaction_id);
        
        if ($response) {
            /*
            log_message('debug', 'Dados recebidos da API: ' . json_encode($response));
            */
            
            $update_data = [
                'status' => $response->status,
                'updated_at' => date('Y-m-d H:i:s'),
                'qr_code' => $response->payload->qrcode_id ?? null,
                'qr_code_text' => $response->payload->data ?? null,
                'qr_code_img' => $response->payload->image ?? null,
                'barcode' => $response->payload->bar_code ?? null,
                'consulta' => '1'
            ];
            
            /*
            log_message('debug', 'Dados para atualização: ' . json_encode($update_data));
            */
            
            $this->ci->db->where('invoice_id', $invoice_id)
                        ->update(db_prefix() . 'pagou_transactions', $update_data);
            
            /*
            log_message('info', 'Transação atualizada com sucesso');
            */
            
            return $this->checkExistingTransaction($invoice_id);
        }
        
        /*
        log_message('error', 'Falha ao atualizar transação - Sem resposta da API');
        */
        return null;
    }

    /**
     * Salva uma nova transação no banco de dados.
     *
     * @param int $invoice_id ID da fatura.
     * @param object $response_data Dados da resposta da API.
     * @return void
     */
    public function saveNewTransaction($invoice_id, $response_data)
    {
        /*
        log_message('info', 'Salvando nova transação para invoice_id: ' . $invoice_id);
        log_message('debug', 'Dados da resposta: ' . json_encode($response_data));
        */
        
        $transaction_data = [
            'invoice_id' => $invoice_id,
            'transaction_id' => $response_data->id ?? '',
            'payment_method' => 'boleto',
            'amount' => $response_data->amount ?? 0,
            'status' => $response_data->status ?? '',
            'date_created' => date('Y-m-d H:i:s'),
            'consulta' => '0'
        ];
        
        /*
        log_message('debug', 'Dados para inserção: ' . json_encode($transaction_data));
        */
        
        $this->ci->db->insert(db_prefix() . 'pagou_transactions', $transaction_data);
        
        /*
        log_message('debug', 'Atualizando status da invoice');
        */
        $this->ci->db->where('id', $invoice_id)
                    ->update(db_prefix() . 'invoices', ['invoice_pagou' => 1]);
        
        /*
        log_message('info', 'Nova transação salva com sucesso');
        */
    }

    /**
     * Lida com os webhooks recebidos da API Pagou.
     *
     * @return void
     */
    public function handleWebhook()
    {
        /*
        log_message('info', 'Recebendo webhook do Pagou');
        */
        
        $payload = file_get_contents('php://input');
        /*
        log_message('debug', 'Payload do webhook: ' . $payload);
        */
        
        $data = json_decode($payload);

        if (!$this->validateWebhookPayload($data)) {
            /*
            log_message('error', 'Webhook inválido - Dados ausentes ou malformados');
            */
            $this->sendResponse(400, 'Invalid webhook data');
            return;
        }

        // Begin database transaction
        $this->ci->db->trans_begin();

        try {
            if ($data->name === 'qrcode.completed') {
                $transaction = $this->findTransaction($data->data->id);
            } else {
                $transaction = $this->findTransactionByMetadata($data->metadata);
            }

            if (!$transaction) {
                /*
                log_message('error', 'Transação não encontrada para ID: ' . ($data->data->id ?? 'N/A'));
                */
                $this->sendResponse(404, 'Transaction not found');
                return;
            }

            $this->updateTransactionWithPayment($transaction, $data);
            $this->updateInvoiceStatus($transaction->invoice_id);

            if ($this->ci->db->trans_status() === FALSE) {
                throw new Exception('Falha na transação do banco de dados');
            }

            $this->ci->db->trans_commit();
            /*
            log_message('info', 'Webhook processado com sucesso');
            */
            $this->sendResponse(200, 'Webhook processed successfully');

        } catch (Exception $e) {
            $this->ci->db->trans_rollback();
            /*
            log_message('error', 'Erro ao processar webhook: ' . $e->getMessage());
            */
            $this->sendResponse(500, 'Error processing webhook');
        }
    }

    /**
     * Valida os dados recebidos no webhook.
     *
     * @param object $data Dados do webhook.
     * @return bool Retorna true se os dados forem válidos, ou false caso contrário.
     */
    private function validateWebhookPayload($data)
    {
        if (!$data) {
            return false;
        }

        if (isset($data->name) && $data->name === 'qrcode.completed') {
            return isset($data->data) && isset($data->data->id) && isset($data->data->amount);
        }

        return isset($data->metadata);
    }

    /**
     * Encontra uma transação com base no ID do pagamento.
     *
     * @param string $paymentId ID do pagamento.
     * @return object|null Retorna o objeto da transação ou null se não encontrado.
     */
    private function findTransaction($paymentId)
    {
        return $this->ci->db->where('transaction_id', $paymentId)
                           ->or_where('qr_code', $paymentId)
                           ->get(db_prefix() . 'pagou_transactions')
                           ->row();
    }

    /**
     * Encontra uma transação com base nos metadados.
     *
     * @param array $metadata Metadados recebidos no webhook.
     * @return object|null Retorna o objeto da transação ou null se não encontrado.
     */
    private function findTransactionByMetadata($metadata)
    {
        $invoice_id = $this->getInvoiceIdFromMetadata($metadata);
        if (!$invoice_id) {
            return null;
        }

        return $this->ci->db->where('invoice_id', $invoice_id)
                           ->get(db_prefix() . 'pagou_transactions')
                           ->row();
    }

    /**
     * Obtém o ID da fatura a partir dos metadados.
     *
     * @param array $metadata Metadados recebidos no webhook.
     * @return int|null Retorna o ID da fatura ou null se não encontrado.
     */
    private function getInvoiceIdFromMetadata($metadata)
    {
        if (!is_array($metadata)) {
            return null;
        }

        foreach ($metadata as $meta) {
            if ($meta->key === 'invoice_id') {
                return $meta->value;
            }
        }
        return null;
    }

    /**
     * Atualiza uma transação com os dados de pagamento recebidos.
     *
     * @param object $transaction Objeto da transação a ser atualizada.
     * @param object $data Dados recebidos no webhook.
     * @return void
     */
    private function updateTransactionWithPayment($transaction, $data)
    {
        $updateData = [
            'status' => 'paid',
            'updated_at' => date('Y-m-d H:i:s'),
            'payment_date' => date('Y-m-d H:i:s')
        ];

        if (isset($data->data->payer)) {
            $updateData['payer_name'] = $data->data->payer->name ?? null;
            $updateData['payer_document'] = $data->data->payer->document ?? null;
            $updateData['bank_info'] = isset($data->data->payer->bank) ? json_encode($data->data->payer->bank) : null;
        }

        $this->ci->db->where('id', $transaction->id)
                     ->update(db_prefix() . 'pagou_transactions', $updateData);

        /*
        log_message('debug', 'Transação atualizada - ID: ' . $transaction->id);
        */
    }

    /**
     * Atualiza o status da fatura para "paga".
     *
     * @param int $invoice_id ID da fatura.
     * @return void
     */
    private function updateInvoiceStatus($invoice_id)
    {
        $this->ci->db->where('id', $invoice_id)
                     ->update(db_prefix() . 'tblinvoices', ['status' => 2]);

        $this->logPaymentActivity($invoice_id);
        /*
        log_message('debug', 'Invoice marcada como paga - ID: ' . $invoice_id);
        */
    }

    /**
     * Registra a atividade de pagamento na fatura.
     *
     * @param int $invoice_id ID da fatura.
     * @return void
     */
    private function logPaymentActivity($invoice_id)
    {
        $description = 'Pagamento recebido via Pagou';
        
        $this->ci->load->model('invoices_model');
        $invoice = $this->ci->invoices_model->get($invoice_id);
        
        if ($invoice) {
            $description .= ' para fatura #' . format_invoice_number($invoice_id);
        }

        /*
        log_activity($description);
        */
    }

    /**
     * Envia uma resposta HTTP com código de status e mensagem.
     *
     * @param int $statusCode Código de status HTTP.
     * @param string $message Mensagem a ser enviada.
     * @return void
     */
    private function sendResponse($statusCode, $message)
    {
        http_response_code($statusCode);
        header('Content-Type: application/json');
        echo json_encode(['message' => $message]);
        exit;
    }
}
?>
