<?php
defined('BASEPATH') or exit('No direct script access allowed');

/* ============================
     Definição da Classe Webhook_handler
   ============================ */
class Webhook_handler
{
    private $ci;
    private $db;

    /**
     * Método Construtor
     *
     * Inicializa o manipulador de webhooks com as dependências necessárias.
     */
    public function __construct()
    {
        $this->ci = &get_instance();
        $this->db = $this->ci->db;
    }

    /**
     * Processa o webhook recebido da API Pagou.
     *
     * @return void
     */
    public function processWebhook()
    {
        // Obter os dados brutos do POST
        $payload = file_get_contents('php://input');
        /*
        log_message('debug', 'Webhook received - Raw payload: ' . $payload);
        */

        // Decodificar o payload JSON
        $data = json_decode($payload);
        if (!$this->validatePayload($data)) {
            /*
            log_message('error', 'Invalid webhook payload received');
            */
            $this->sendResponse(400, 'Invalid webhook payload');
            return;
        }

        // Processar com base no nome do evento
        if ($data->name === 'qrcode.completed') {
            $this->handleQrCodePayment($data->data);
        } else {
            /*
            log_message('warning', 'Unhandled webhook event: ' . $data->name);
            */
            $this->sendResponse(200, 'Event type not handled');
            return;
        }
    }

    /**
     * Valida os dados recebidos no webhook.
     *
     * @param object $data Dados do webhook.
     * @return bool Retorna true se os dados forem válidos, ou false caso contrário.
     */
    private function validatePayload($data)
    {
        if (!$data || !isset($data->name) || !isset($data->data)) {
            return false;
        }

        if (isset($data->name) && $data->name === 'qrcode.completed') {
            return isset($data->data->id) && isset($data->data->amount);
        }

        return isset($data->metadata);
    }

    /**
     * Lida com pagamentos via QR Code.
     *
     * @param object $paymentData Dados do pagamento recebidos no webhook.
     * @return void
     */
    private function handleQrCodePayment($paymentData)
    {
        /*
        log_message('info', 'Processing QR code payment - ID: ' . $paymentData->id);
        */

        // Encontrar a transação no banco de dados
        $transaction = $this->findTransaction($paymentData->id);
        if (!$transaction) {
            /*
            log_message('error', 'Transaction not found for ID: ' . $paymentData->id);
            */
            $this->sendResponse(404, 'Transaction not found');
            return;
        }

        // Iniciar transação no banco de dados
        $this->db->trans_begin();

        try {
            // Atualizar status da transação
            $this->updateTransactionStatus($transaction, $paymentData);

            // Atualizar status da fatura
            $this->updateInvoiceStatus($transaction->invoice_id);

            // Verificar se a transação no banco de dados teve sucesso
            if ($this->db->trans_status() === FALSE) {
                throw new Exception('Falha na transação do banco de dados');
            }

            // Confirmar transação no banco de dados
            $this->db->trans_commit();
            /*
            log_message('info', 'Payment processed successfully for invoice ID: ' . $transaction->invoice_id);
            */
            $this->sendResponse(200, 'Payment processed successfully');

        } catch (Exception $e) {
            // Reverter transação no banco de dados em caso de erro
            $this->db->trans_rollback();
            /*
            log_message('error', 'Error processing payment: ' . $e->getMessage());
            */
            $this->sendResponse(500, 'Error processing payment');
        }
    }

    /**
     * Encontra uma transação com base no ID do pagamento.
     *
     * @param string $paymentId ID do pagamento.
     * @return object|null Retorna o objeto da transação ou null se não encontrado.
     */
    private function findTransaction($paymentId)
    {
        return $this->db->where('transaction_id', $paymentId)
                        ->or_where('qr_code', $paymentId)
                        ->get(db_prefix() . 'pagou_transactions')
                        ->row();
    }

    /**
     * Atualiza o status de uma transação com os dados recebidos no webhook.
     *
     * @param object $transaction Objeto da transação a ser atualizada.
     * @param object $paymentData Dados do pagamento recebidos no webhook.
     * @return void
     */
    private function updateTransactionStatus($transaction, $paymentData)
    {
        $updateData = [
            'status' => 'paid',
            'updated_at' => date('Y-m-d H:i:s'),
            'payment_date' => date('Y-m-d H:i:s'),
            'payer_name' => $paymentData->payer->name,
            'payer_document' => $paymentData->payer->document,
            'bank_info' => json_encode($paymentData->payer->bank)
        ];

        $this->db->where('id', $transaction->id)
                 ->update(db_prefix() . 'pagou_transactions', $updateData);

        /*
        log_message('debug', 'Transaction updated - ID: ' . $transaction->id);
        */
    }

    /**
     * Atualiza o status da fatura para "paga".
     *
     * @param int $invoiceId ID da fatura.
     * @return void
     */
    private function updateInvoiceStatus($invoiceId)
    {
        // Atualizar status da fatura para paga (2)
        $this->db->where('id', $invoiceId)
                 ->update(db_prefix() . 'tblinvoices', ['status' => 2]);

        // Registrar atividade de pagamento
        $this->logPaymentActivity($invoiceId);

        /*
        log_message('debug', 'Invoice marcada como paga - ID: ' . $invoiceId);
        */
    }

    /**
     * Registra a atividade de pagamento na fatura.
     *
     * @param int $invoiceId ID da fatura.
     * @return void
     */
    private function logPaymentActivity($invoiceId)
    {
        $staffId = 0; // Ação automatizada do sistema
        $description = 'Invoice payment received via Pagou QR Code';
        
        $this->ci->load->model('invoices_model');
        $invoice = $this->ci->invoices_model->get($invoiceId);
        
        if ($invoice) {
            $description .= ' for invoice #' . format_invoice_number($invoiceId);
        }

        /*
        log_activity($description, $staffId);
        */
    }

    /**
     * Envia uma resposta HTTP com código de status e mensagem.
     *
     * @param int $statusCode Código de status HTTP.
     * @param string $message Mensagem a ser enviada.
     * @return void
     */
    private function sendResponse($statusCode, $message)
    {
        http_response_code($statusCode);
        header('Content-Type: application/json');
        echo json_encode(['message' => $message]);
        exit;
    }
}
?>
